<?php
/**
 * Plugin Name: IndexNow URL Submitter
 * Plugin URI: https://example.com/plugins/indexnow-url-submitter
 * Description: Submit multiple URLs to search engines using the IndexNow protocol
 * Version: 1.0.0
 * Author: Claude
 * Author URI: https://example.com
 * License: GPL v2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: indexnow-url-submitter
 */

// If this file is called directly, abort.
if (!defined('WPINC')) {
    die;
}

class IndexNow_URL_Submitter {

    private $plugin_name = 'indexnow-url-submitter';
    private $version = '1.0.0';

    /**
     * Initialize the plugin.
     */
    public function __construct() {
        add_action('admin_menu', array($this, 'add_admin_menu'));
        add_action('admin_init', array($this, 'register_settings'));
        add_action('save_post', array($this, 'submit_post_url'), 10, 3);
        register_activation_hook(__FILE__, array($this, 'activate'));
    }

    /**
     * Plugin activation tasks.
     */
    public function activate() {
        $this->create_key_file();
    }

    /**
     * Create the API key file on activation.
     */
    private function create_key_file() {
        $api_key = get_option('indexnow_api_key');
        
        // Generate a new API key if one doesn't exist
        if (empty($api_key)) {
            $api_key = $this->generate_api_key();
            update_option('indexnow_api_key', $api_key);
        }
        
        // Try to create the key file in the WordPress root directory
        $file_path = ABSPATH . $api_key . '.txt';
        @file_put_contents($file_path, $api_key);
        
        // Also create a key file in the uploads directory as a backup
        $upload_dir = wp_upload_dir();
        $key_location = $upload_dir['basedir'] . '/indexnow-key.txt';
        @file_put_contents($key_location, $api_key);
        update_option('indexnow_key_location', $upload_dir['baseurl'] . '/indexnow-key.txt');
    }

    /**
     * Generate a random API key.
     */
    private function generate_api_key() {
        return bin2hex(random_bytes(16)); // 32 character key
    }

    /**
     * Add admin menu.
     */
    public function add_admin_menu() {
        add_options_page(
            'IndexNow URL Submitter',
            'IndexNow URL Submitter',
            'manage_options',
            $this->plugin_name,
            array($this, 'display_admin_page')
        );
    }

    /**
     * Register plugin settings.
     */
    public function register_settings() {
        register_setting($this->plugin_name, 'indexnow_api_key');
        register_setting($this->plugin_name, 'indexnow_key_location');
        register_setting($this->plugin_name, 'indexnow_auto_submit');
        register_setting($this->plugin_name, 'indexnow_post_types');
    }

    /**
     * Display the admin page.
     */
    public function display_admin_page() {
        $api_key = get_option('indexnow_api_key', '');
        $key_location = get_option('indexnow_key_location', '');
        $auto_submit = get_option('indexnow_auto_submit', '1');
        $selected_post_types = get_option('indexnow_post_types', array('post', 'page'));
        
        // Get all available post types
        $post_types = get_post_types(array('public' => true), 'objects');
        ?>
        <div class="wrap">
            <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
            
            <form method="post" action="options.php">
                <?php
                settings_fields($this->plugin_name);
                do_settings_sections($this->plugin_name);
                ?>
                
                <table class="form-table">
                    <tr>
                        <th scope="row">
                            <label for="indexnow_api_key">API Key</label>
                        </th>
                        <td>
                            <input type="text" id="indexnow_api_key" name="indexnow_api_key" 
                                value="<?php echo esc_attr($api_key); ?>" class="regular-text">
                            <p class="description">
                                Your IndexNow API Key. If changed, make sure to update the key file.
                            </p>
                        </td>
                    </tr>
                    
                    <tr>
                        <th scope="row">
                            <label for="indexnow_key_location">Key File Location</label>
                        </th>
                        <td>
                            <input type="text" id="indexnow_key_location" name="indexnow_key_location" 
                                value="<?php echo esc_attr($key_location); ?>" class="regular-text">
                            <p class="description">
                                URL to your key file. Default is in the uploads directory.
                            </p>
                        </td>
                    </tr>
                    
                    <tr>
                        <th scope="row">Auto Submit</th>
                        <td>
                            <label for="indexnow_auto_submit">
                                <input type="checkbox" id="indexnow_auto_submit" name="indexnow_auto_submit" 
                                    value="1" <?php checked('1', $auto_submit); ?>>
                                Automatically submit URLs when content is published or updated
                            </label>
                        </td>
                    </tr>
                    
                    <tr>
                        <th scope="row">Post Types to Submit</th>
                        <td>
                            <?php foreach ($post_types as $post_type) : ?>
                                <label for="indexnow_post_types_<?php echo $post_type->name; ?>">
                                    <input type="checkbox" id="indexnow_post_types_<?php echo $post_type->name; ?>" 
                                        name="indexnow_post_types[]" value="<?php echo $post_type->name; ?>"
                                        <?php checked(in_array($post_type->name, (array)$selected_post_types)); ?>>
                                    <?php echo $post_type->label; ?>
                                </label><br>
                            <?php endforeach; ?>
                        </td>
                    </tr>
                </table>
                
                <?php submit_button(); ?>
            </form>
            
            <hr>
            
            <h2>Submit URLs Manually</h2>
            <form method="post" action="">
                <?php wp_nonce_field('indexnow_submit_urls', 'indexnow_nonce'); ?>
                <table class="form-table">
                    <tr>
                        <th scope="row">
                            <label for="indexnow_urls">URLs to Submit</label>
                        </th>
                        <td>
                            <textarea id="indexnow_urls" name="indexnow_urls" rows="10" class="large-text"
                                placeholder="https://example.com/page1&#10;https://example.com/page2&#10;https://example.com/page3"></textarea>
                            <p class="description">
                                Enter one URL per line. All URLs must be from the same domain.
                            </p>
                        </td>
                    </tr>
                </table>
                
                <input type="submit" name="indexnow_submit" class="button button-primary" value="Submit URLs">
            </form>
            
            <?php
            // Handle manual URL submission
            if (isset($_POST['indexnow_submit']) && isset($_POST['indexnow_nonce']) && 
                wp_verify_nonce($_POST['indexnow_nonce'], 'indexnow_submit_urls')) {
                
                $urls = isset($_POST['indexnow_urls']) ? explode("\n", $_POST['indexnow_urls']) : array();
                $urls = array_map('trim', $urls);
                $urls = array_filter($urls);
                
                if (!empty($urls)) {
                    $result = $this->submit_urls($urls);
                    
                    if ($result === true) {
                        echo '<div class="notice notice-success is-dismissible"><p>URLs submitted successfully!</p></div>';
                    } else {
                        echo '<div class="notice notice-error is-dismissible"><p>Error submitting URLs: ' . esc_html($result) . '</p></div>';
                    }
                } else {
                    echo '<div class="notice notice-error is-dismissible"><p>No valid URLs provided.</p></div>';
                }
            }
            ?>
        </div>
        <?php
    }

    /**
     * Submit post URL when it's published or updated.
     */
    public function submit_post_url($post_id, $post, $update) {
        // Check if auto-submit is enabled
        if (get_option('indexnow_auto_submit', '1') !== '1') {
            return;
        }
        
        // Don't submit if this is an autosave or revision
        if (wp_is_post_autosave($post_id) || wp_is_post_revision($post_id)) {
            return;
        }
        
        // Check post status
        if ($post->post_status !== 'publish') {
            return;
        }
        
        // Check post type
        $selected_post_types = get_option('indexnow_post_types', array('post', 'page'));
        if (!in_array($post->post_type, (array)$selected_post_types)) {
            return;
        }
        
        // Submit the URL
        $url = get_permalink($post_id);
        $this->submit_urls(array($url));
    }

    /**
     * Submit URLs to IndexNow API.
     */
    public function submit_urls($urls) {
        if (empty($urls)) {
            return 'No URLs provided';
        }
        
        $api_key = get_option('indexnow_api_key', '');
        if (empty($api_key)) {
            return 'API key not configured';
        }
        
        $key_location = get_option('indexnow_key_location', '');
        if (empty($key_location)) {
            $key_location = home_url('/' . $api_key . '.txt');
        }
        
        // Get domain from the first URL
        $host = parse_url($urls[0], PHP_URL_HOST);
        
        // Make sure all URLs are from the same domain
        foreach ($urls as $url) {
            if (parse_url($url, PHP_URL_HOST) !== $host) {
                return 'All URLs must be from the same domain';
            }
        }
        
        // Prepare the request payload
        $payload = array(
            'host' => $host,
            'key' => $api_key,
            'keyLocation' => $key_location,
            'urlList' => array_values($urls)
        );
        
        // Send the request
        $response = wp_remote_post('https://api.indexnow.org/indexnow', array(
            'headers' => array(
                'Content-Type' => 'application/json; charset=utf-8'
            ),
            'body' => json_encode($payload),
            'timeout' => 30
        ));
        
        if (is_wp_error($response)) {
            return $response->get_error_message();
        }
        
        $response_code = wp_remote_retrieve_response_code($response);
        if ($response_code !== 200) {
            return 'API returned status code: ' . $response_code;
        }
        
        return true;
    }
}

// Initialize the plugin
$indexnow_url_submitter = new IndexNow_URL_Submitter();
